#include "lifecycle.h"

typedef struct appdata {
    Evas_Object *win;
    Evas_Object *conform;
    //Evas_Object *label;
    Evas_Object *entry;
} appdata_s;

static void
show_message(appdata_s* ad, const char* msg)
{
    dlog_print(DLOG_ERROR, "tag", msg);
    const char *old_msg = elm_object_text_get(ad->entry);
    const int len = strlen(old_msg) + strlen(msg) + sizeof("<br/>");
    char *total_msg = malloc(len);
    snprintf(total_msg, len, "%s%s<br/>", old_msg, msg);
    elm_object_text_set(ad->entry, total_msg);
    elm_entry_cursor_pos_set(ad->entry, len - 1);
    free(total_msg);
}

static void
win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = data;
	show_message(ad, "win_back_cb()");
	/* Let window go to hide state. */
	elm_win_iconified_set(ad->win, EINA_TRUE);
}

static void
create_base_gui(appdata_s *ad)
{
	/* set up policy to exit when last window is closed */
	elm_policy_set(ELM_POLICY_QUIT, ELM_POLICY_QUIT_LAST_WINDOW_CLOSED);
	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	int rots[4] = { 0, 90, 180, 270 };
	elm_win_wm_rotation_available_rotations_set(ad->win, (const int *)(&rots), 4);

	eext_object_event_callback_add(ad->win, EEXT_CALLBACK_BACK, win_back_cb, ad);

    /* Conformant */
    ad->conform = elm_conformant_add(ad->win);
    elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
    elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
    evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(ad->win, ad->conform);
    evas_object_show(ad->conform);

    /* Label*/
    //ad->label = elm_label_add(ad->conform);
    //elm_object_text_set(ad->label, "<align=center>Hello EFL</align>");
    //evas_object_size_hint_weight_set(ad->label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    //elm_object_content_set(ad->conform, ad->label);

    /* Entry */
    ad->entry = elm_entry_add(ad->conform);
    elm_entry_scrollable_set(ad->entry, EINA_TRUE);
    elm_entry_editable_set(ad->entry, EINA_FALSE);
    evas_object_size_hint_weight_set(ad->entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_object_content_set(ad->conform, ad->entry);
    evas_object_show(ad->entry);

    /* Show window after base gui is set up */
    evas_object_show(ad->win);
}

static bool
app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);
	show_message(ad, "app_create()");
	return true;
}

static void
app_control(app_control_h app_control, void *data)
{
    appdata_s *ad = data;
    show_message(ad, "app_control()");
}

static void
app_pause(void *data)
{
    appdata_s *ad = data;
    show_message(ad, "app_pause()");
}

static void
app_resume(void *data)
{
    appdata_s *ad = data;
    show_message(ad, "app_resume()");
}

static void
app_terminate(void *data)
{
    appdata_s *ad = data;
    show_message(ad, "app_terminate()");
}

static void
ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

int
main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "app_main() is failed. err = %d", ret);
	}

	return ret;
}
